@@ -42,6 +42,7 @@ mod tests {
 
         assert_eq!(c.calculate_line("1 + 2"), Ok(3.));
         assert_eq!(c.calculate_line("-2 * -2"), Ok(4.));
+        assert_eq!(c.calculate_line("sqrt(sqrt(16)) + 10"), Ok(12.));
     }
 
     #[test]
@@ -50,7 +51,7 @@ mod tests {
 
         assert_eq!(
             c.calculate_line("1 / 0"),
-            Err(Error::Eval(EvalError::ZeroDivision))
+            Err(Error::Eval(EvalError::DivisionByZero))
         );
     }
 
